"""
Make a directory
----------------
"""
from XRootD import client
from XRootD.client.flags import MkDirFlags

myclient = client.FileSystem("root://localhost")
myclient.mkdir("/tmp/some/dir", MkDirFlags.MAKEPATH)



from setuptools import setup
from distutils.core import Extension
from distutils import sysconfig
from os import getenv, walk, path, path, getcwd, chdir
from platform import system
import subprocess

# Remove the "-Wstrict-prototypes" compiler option, which isn't valid for C++.
cfg_vars = sysconfig.get_config_vars()
opt = cfg_vars["OPT"]
cfg_vars["OPT"] = " ".join( flag for flag in opt.split() if flag != '-Wstrict-prototypes' )


sources = list()
depends = list()

for dirname, dirnames, filenames in walk('src'):
  for filename in filenames:
    if filename.endswith('.cc'):
      sources.append(path.join(dirname, filename))
    elif filename.endswith('.hh'):
      depends.append(path.join(dirname, filename))

# Get package version
with open ('VERSION_INFO') as verfile:
    version = verfile.read().strip()

def getincdir_osx():
    # Assume xrootd was installed via homebrew
    return '/usr/local/Cellar/xrootd/{0}/include/xrootd'.format(version)

def getlibdir():
    return (system() == 'Darwin' and '/usr/local/lib') or '/usr/lib'

def getincdir():
    return (system() == 'Darwin' and getincdir_osx()) or '/usr/include/xrootd'

xrdlibdir = getenv( 'XRD_LIBDIR' ) or getlibdir()
xrdincdir = getenv( 'XRD_INCDIR' ) or getincdir()

print 'XRootD library dir: ', xrdlibdir
print 'XRootD include dir: ', xrdincdir
print 'Version:            ', version

setup( name             = 'pyxrootd',
       version          = version,
       author           = 'XRootD Developers',
       author_email     = 'xrootd-dev@slac.stanford.edu',
       url              = 'http://xrootd.org',
       license          = 'LGPLv3+',
       description      = "XRootD Python bindings",
       long_description = "XRootD Python bindings",
       packages         = ['pyxrootd', 'XRootD', 'XRootD.client'],
       package_dir      = {'pyxrootd'     : 'src',
                           'XRootD'       : 'libs',
                           'XRootD.client': 'libs/client'},
       ext_modules      = [
           Extension(
               'pyxrootd.client',
               sources   = sources,
               depends   = depends,
               libraries = ['XrdCl', 'XrdUtils', 'dl'],
               extra_compile_args = ['-g'],
               include_dirs = [xrdincdir],
               library_dirs = [xrdlibdir]
               )
           ]
       )



SERVER_URL  = 'root://localhost/'
smallfile   = SERVER_URL + '/tmp/spam'
smallcopy   = SERVER_URL + '/tmp/eggs'
smallbuffer = 'gre\0en\neggs\nand\nham\n'
bigfile     = SERVER_URL + '/tmp/bigfile'
bigcopy     = SERVER_URL + '/tmp/bigcopy'


"""
Iterate over a file, delimited by newline characters
----------------------------------------------------

Produces the following output::

  'green\n'
  'eggs\n'
  'and\n'
  'spam\n'

"""
from XRootD import client

with client.File() as f:
  f.open('root://localhost//tmp/eggs')

  for line in f:
    print '%r' % line


from __future__ import absolute_import, division, print_function

from .filesystem import FileSystem as FileSystem
from .file import File as File
from .url import URL as URL
from .copyprocess import CopyProcess as CopyProcess



"""
Ask the server for some information
-----------------------------------

Produces output similar to the following::

  oss.cgroup=public&oss.space=52844687360&oss.free=27084992512&oss.maxf=27084992512&oss.used=25759694848&oss.quota=-1

For more information about XRootD query codes and arguments, see
`the relevant section in the protocol reference
<http://xrootd.slac.stanford.edu/doc/prod/XRdv299.htm#_Toc337053385>`_.
"""
from XRootD import client
from XRootD.client.flags import QueryCode

myclient = client.FileSystem("root://localhost")
status, response = myclient.query(QueryCode.SPACE, '/tmp')

print response


#-------------------------------------------------------------------------------
# Copyright (c) 2012-2013 by European Organization for Nuclear Research (CERN)
# Author: Justin Salmon <jsalmon@cern.ch>
#-------------------------------------------------------------------------------
# XRootD is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# XRootD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with XRootD.  If not, see <http:#www.gnu.org/licenses/>.
#-------------------------------------------------------------------------------
from __future__ import absolute_import, division, print_function

def enum(**enums):
  """Build the equivalent of a C++ enum"""
  reverse = dict((value, key) for key, value in enums.items())
  enums['reverse_mapping'] = reverse
  return type('Enum', (), enums)

QueryCode = enum(
  STATS          = 1,
  PREPARE        = 2,
  CHECKSUM       = 3,
  XATTR          = 4,
  SPACE          = 5,
  CHECKSUMCANCEL = 6,
  CONFIG         = 7,
  VISA           = 8,
  OPAQUE         = 16,
  OPAQUEFILE     = 32
)

OpenFlags = enum(
  NONE      = 0,
# COMPRESS  = 1,
  DELETE    = 2,
  FORCE     = 4,
  NEW       = 8,
  READ      = 16,
  UPDATE    = 32,
# ASYNC     = 64,
  REFRESH   = 128,
  MAKEPATH  = 256,
  APPEND    = 512,
# RETSTAT   = 1024,
  REPLICA   = 2048,
  POSC      = 4096,
  NOWAIT    = 8192,
  SEQIO     = 16384
)

AccessMode = enum(
  NONE = 0,
  UR   = 0x100,
  UW   = 0x080,
  UX   = 0x040,
  GR   = 0x020,
  GW   = 0x010,
  GX   = 0x008,
  OR   = 0x004,
  OW   = 0x002,
  OX   = 0x001
)

MkDirFlags = enum(
  NONE     = 0,
  MAKEPATH = 1
)

DirListFlags = enum(
  NONE   = 0,
  STAT   = 1,
  LOCATE = 2
)

PrepareFlags = enum(
# CANCEL    = 1,
# NOTIFY    = 2,
# NOERRS    = 4,
  STAGE     = 8,
  WRITEMODE = 16,
  COLOCATE  = 32,
  FRESH     = 64
)

HostTypes = enum(
  IS_MANAGER = 0x00000002,
  IS_SERVER  = 0x00000001,
  ATTR_META  = 0x00000100,
  ATTR_PROXY = 0x00000200,
  ATTR_SUPER = 0x00000400
)

StatInfoFlags = enum(
  X_BIT_SET    = 1,
  IS_DIR       = 2,
  OTHER        = 4,
  OFFLINE      = 8,
  IS_READABLE  = 16,
  IS_WRITABLE  = 32,
  POSC_PENDING = 64,
  BACKUP_EXISTS = 128
)

LocationType = enum(
  MANAGER_ONLINE  = 0,
  MANAGER_PENDING = 1,
  SERVER_ONLINE   = 2,
  SERVER_PENDING  = 3
)

AccessType = enum(
  READ       = 0,
  READ_WRITE = 1
)



from XRootD import client
import pytest, sys
from env import *

def test_creation():
  u = client.FileSystem(SERVER_URL).url
  assert u is not None

def test_deletion():
  u = client.FileSystem(SERVER_URL).url
  del u

  if sys.hexversion > 0x03000000:
    pytest.raises(UnboundLocalError, 'assert u')
  else:
    pytest.raises(NameError, 'assert u')

def test_valid():
  u = client.FileSystem(SERVER_URL).url
  assert u.is_valid()

def test_invalid():
  u = client.FileSystem('root://').url
  assert u.is_valid() == False

def test_getters():
  u = client.FileSystem("root://user1:passwd1@host1:123//path?param1=val1&param2=val2").url
  assert u.is_valid()
  assert u.hostid == 'user1:passwd1@host1:123'
  assert u.protocol == 'root'
  assert u.username == 'user1'
  assert u.password == 'passwd1'
  assert u.hostname == 'host1'
  assert u.port == 123
  assert u.path == '/path'
  assert u.path_with_params == '/path?param1=val1&param2=val2'

def test_setters():
  u = client.FileSystem(SERVER_URL).url
  u.protocol = 'root'
  assert u.protocol == 'root'
  u.username = 'user1'
  assert u.username == 'user1'
  u.password = 'passwd1'
  assert u.password == 'passwd1'
  u.hostname = 'host1'
  assert u.hostname == 'host1'
  u.port = 123
  assert u.port == 123
  u.path = '/path'
  assert u.path == '/path'
  u.clear()
  assert str(u) == ''



from setuptools import setup, Extension
from setuptools.command.install import install

import subprocess
import sys


class CustomInstall(install):
    def run(self): 
        subprocess.call('./install.sh')


def get_version():
    version = subprocess.check_output(['./genversion.sh', '--print-only'])
    if version.startswith('v'):
        version = version[1:]
    version = version.split('-')[0]
    return version + '/1'

version = get_version()

with open('bindings/python/VERSION_INFO', 'w') as vi:
    vi.write(version)


setup( 
    name             = 'kjpyxrootd',
    version          = version,
    author           = 'XRootD Developers',
    author_email     = 'xrootd-dev@slac.stanford.edu',
    url              = 'http://xrootd.org',
    license          = 'LGPLv3+',
    description      = "XRootD with Python bindings",
    long_description = "XRootD with Python bindings",
    cmdclass        = {'install': CustomInstall}
)



#-------------------------------------------------------------------------------
# Copyright (c) 2012-2013 by European Organization for Nuclear Research (CERN)
# Author: Justin Salmon <jsalmon@cern.ch>
#-------------------------------------------------------------------------------
# XRootD is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# XRootD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with XRootD.  If not, see <http:#www.gnu.org/licenses/>.
#-------------------------------------------------------------------------------
from __future__ import absolute_import, division, print_function

from pyxrootd import client

class URL(object):
  """Server URL object.

  This class has each portion of an `XRootD` URL split up as attributes. For
  example, given the URL::

    >>> url = URL(root://user1:passwd1@host1:1234//path?param1=val1&param2=val2)

  then ``url.hostid`` would return `user1:passwd1@host1:1234`.

  :var           hostid: The host part of the URL, i.e. ``user1:passwd1@host1:1234``
  :var         protocol: The protocol part of the URL, i.e. ``root``
  :var         username: The username part of the URL, i.e. ``user1``
  :var         password: The password part of the URL, i.e. ``passwd1``
  :var         hostname: The name of the target host part of the URL, i.e. ``host1``
  :var             port: The target port part of the URL, i.e. ``1234``
  :var             path: The path part of the URL, i.e. ``path``
  :var path_with_params: The path part of the URL with parameters, i.e.
                         ``path?param1=val1&param2=val2``
  """

  def __init__(self, url):
    self.__url = client.URL(url)

  def __str__(self):
    return str(self.__url)

  @property
  def hostid(self):
    return self.__url.hostid

  @property
  def protocol(self):
    return self.__url.protocol

  @property
  def username(self):
    return self.__url.username

  @property
  def password(self):
    return self.__url.password

  @property
  def hostname(self):
    return self.__url.hostname

  @property
  def port(self):
    return self.__url.port

  @property
  def path(self):
    return self.__url.path

  @property
  def path_with_params(self):
    return self.__url.path_with_params

  def is_valid(self):
    """Return the validity of the URL"""
    return self.__url.is_valid()

  def clear(self):
    """Clear the URL"""
    return self.__url.clear()



"""
Add a number of copy jobs and run them in parallel with a progress handler
--------------------------------------------------------------------------

Produces output similar to the following::

  id: 1, total: 4
  source: /tmp/spam
  target: /tmp/spam1
  processed: 20, total: 20
  end status: [SUCCESS] 
  id: 2, total: 4
  source: /tmp/spam
  target: root://localhost//tmp/spam2
  processed: 20, total: 20
  end status: [SUCCESS] 
  id: 3, total: 4
  source: root://localhost//tmp/spam
  target: /tmp/spam3
  processed: 20, total: 20
  end status: [SUCCESS] 
  id: 4, total: 4
  source: root://localhost//tmp/spam
  target: root://localhost//tmp/spam4
  processed: 20, total: 20
  end status: [SUCCESS] 

"""
from XRootD import client

class MyCopyProgressHandler(client.utils.CopyProgressHandler):
  def begin(self, jobId, total, source, target):
    print 'id: %d, total: %d' % (jobId, total)
    print 'source: %s' % source
    print 'target: %s' % target

  def end(self, jobId, result):
    print 'end status:', jobId, result

  def update(self, jobId, processed, total):
    print 'jobId: %d, processed: %d, total: %d' % (jobId, processed, total)

  def should_cancel( jobId ):
    return False

process = client.CopyProcess()

# From local to local
process.add_job( '/tmp/spam', '/tmp/spam1' )
# From local to remote
process.add_job( '/tmp/spam', 'root://localhost//tmp/spam2' )
# From remote to local
process.add_job( 'root://localhost//tmp/spam', '/tmp/spam3' )
# From remote to remote
process.add_job( 'root://localhost//tmp/spam', 'root://localhost//tmp/spam4' )

handler = MyCopyProgressHandler()
process.prepare()
process.run(handler)



"""
Ask a for a directory listing
-----------------------------

Produces output similar to the following::

  2013-04-12 09:46:51         20 spam
  2013-04-05 08:23:00       4096 .xrootd
  2013-04-12 09:33:25         20 eggs
"""

from XRootD import client
from XRootD.client.flags import DirListFlags

myclient = client.FileSystem('root://localhost')
status, listing = myclient.dirlist('/tmp', DirListFlags.STAT)

print listing.parent
for entry in listing:
  print "{0} {1:>10} {2}".format(entry.statinfo.modtimestr, entry.statinfo.size, entry.name)